@@ -1,3 +1,5 @@
+//! Bit vector in a plain format, supporting some utilities such as update, chunking, and predecessor queries.
+pub mod iter;
 pub mod unary;
 
 use std::io::{Read, Write};
@@ -6,8 +8,9 @@ use std::mem::size_of;
 use anyhow::Result;
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
 
-use crate::bit_vector::unary::UnaryIterator;
-use crate::{broadword, util};
+use crate::bit_vector::iter::Iter;
+use crate::bit_vector::unary::UnaryIter;
+use crate::{broadword, util, Searial};
 
 pub(crate) const WORD_LEN: usize = std::mem::size_of::<usize>() * 8;
 
@@ -20,7 +23,7 @@ pub(crate) const WORD_LEN: usize = std::mem::size_of::<usize>() * 8;
 /// ```
 /// use sucds::BitVector;
 ///
-/// let bv = BitVector::from_bits(&[true, false, false, true]);
+/// let bv = BitVector::from_bits([true, false, false, true]);
 ///
 /// assert_eq!(bv.get_bit(0), true);
 /// assert_eq!(bv.get_bit(1), false);
@@ -31,13 +34,6 @@ pub(crate) const WORD_LEN: usize = std::mem::size_of::<usize>() * 8;
 /// assert_eq!(bv.predecessor0(2), Some(2));
 /// assert_eq!(bv.successor1(1), Some(3));
 /// assert_eq!(bv.successor0(1), Some(1));
-///
-/// let mut bytes = vec![];
-/// let size = bv.serialize_into(&mut bytes).unwrap();
-/// let other = BitVector::deserialize_from(&bytes[..]).unwrap();
-/// assert_eq!(bv, other);
-/// assert_eq!(size, bytes.len());
-/// assert_eq!(size, bv.size_in_bytes());
 /// ```
 #[derive(Default, Clone, PartialEq, Eq)]
 pub struct BitVector {
@@ -67,34 +63,6 @@ impl BitVector {
         }
     }
 
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
-    ///
-    /// # Arguments
-    ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
-        let mem = util::vec_io::serialize_usize(&self.words, &mut writer)?;
-        writer.write_u64::<LittleEndian>(self.len as u64)?;
-        Ok(mem + size_of::<u64>())
-    }
-
-    /// Deserializes the data structure from the reader.
-    ///
-    /// # Arguments
-    ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
-        let words = util::vec_io::deserialize_usize(&mut reader)?;
-        let len = reader.read_u64::<LittleEndian>()? as usize;
-        Ok(Self { words, len })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        util::vec_io::size_in_bytes(&self.words) + size_of::<u64>()
-    }
-
     /// Creates a new [`BitVector`] from input bitset `bits`.
     ///
     /// # Arguments
@@ -106,18 +74,18 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[true, false, false, true]);
+    /// let bv = BitVector::from_bits([true, false, false, true]);
     /// assert_eq!(bv.get_bit(0), true);
     /// assert_eq!(bv.get_bit(1), false);
     /// assert_eq!(bv.get_bit(2), false);
     /// assert_eq!(bv.get_bit(3), true);
     /// ```
-    pub fn from_bits<'a, I>(bits: I) -> Self
+    pub fn from_bits<I>(bits: I) -> Self
     where
-        I: IntoIterator<Item = &'a bool>,
+        I: IntoIterator<Item = bool>,
     {
         let mut this = Self::new();
-        bits.into_iter().for_each(|&b| this.push_bit(b));
+        bits.into_iter().for_each(|b| this.push_bit(b));
         this
     }
 
@@ -132,7 +100,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[true, false, false, true]);
+    /// let bv = BitVector::from_bits([true, false, false, true]);
     /// assert_eq!(bv.get_bit(0), true);
     /// assert_eq!(bv.get_bit(1), false);
     /// assert_eq!(bv.get_bit(2), false);
@@ -157,7 +125,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let mut bv = BitVector::from_bits(&[false, true, true, false]);
+    /// let mut bv = BitVector::from_bits([false, true, true, false]);
     /// bv.set_bit(0, true);
     /// bv.set_bit(2, false);
     /// assert_eq!(bv.get_bit(0), true);
@@ -215,7 +183,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[true, false, true, false, true]);
+    /// let bv = BitVector::from_bits([true, false, true, false, true]);
     /// assert_eq!(bv.get_bits(1, 4), 0b1010);
     /// ```
     #[inline(always)]
@@ -334,7 +302,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[false, true, false, true]);
+    /// let bv = BitVector::from_bits([false, true, false, true]);
     /// assert_eq!(bv.predecessor1(3), Some(3));
     /// assert_eq!(bv.predecessor1(2), Some(1));
     /// assert_eq!(bv.predecessor1(1), Some(1));
@@ -369,7 +337,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[true, false, true, false]);
+    /// let bv = BitVector::from_bits([true, false, true, false]);
     /// assert_eq!(bv.successor1(0), Some(0));
     /// assert_eq!(bv.successor1(1), Some(2));
     /// assert_eq!(bv.successor1(2), Some(2));
@@ -405,7 +373,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[true, false, true, false]);
+    /// let bv = BitVector::from_bits([true, false, true, false]);
     /// assert_eq!(bv.predecessor0(3), Some(3));
     /// assert_eq!(bv.predecessor0(2), Some(1));
     /// assert_eq!(bv.predecessor0(1), Some(1));
@@ -440,7 +408,7 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[false, true, false, true]);
+    /// let bv = BitVector::from_bits([false, true, false, true]);
     /// assert_eq!(bv.successor0(0), Some(0));
     /// assert_eq!(bv.successor0(1), Some(2));
     /// assert_eq!(bv.successor0(2), Some(2));
@@ -464,6 +432,27 @@ impl BitVector {
         }
     }
 
+    /// Creates an iterator for enumerating bits.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::BitVector;
+    ///
+    /// let bv = BitVector::from_bits([false, true, false, false, true]);
+    /// let mut it = bv.iter();
+    ///
+    /// assert_eq!(it.next(), Some(false));
+    /// assert_eq!(it.next(), Some(true));
+    /// assert_eq!(it.next(), Some(false));
+    /// assert_eq!(it.next(), Some(false));
+    /// assert_eq!(it.next(), Some(true));
+    /// assert_eq!(it.next(), None);
+    /// ```
+    pub const fn iter(&self) -> Iter {
+        Iter::new(self)
+    }
+
     /// Creates an iterator for enumerating positions of set bits.
     ///
     /// # Examples
@@ -471,15 +460,15 @@ impl BitVector {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[false, true, false, false, true]);
+    /// let bv = BitVector::from_bits([false, true, false, false, true]);
     /// let mut it = bv.unary_iter(1);
     ///
     /// assert_eq!(it.next(), Some(1));
     /// assert_eq!(it.next(), Some(4));
     /// assert_eq!(it.next(), None);
     /// ```
-    pub fn unary_iter(&self, pos: usize) -> UnaryIterator {
-        UnaryIterator::new(self, pos)
+    pub fn unary_iter(&self, pos: usize) -> UnaryIter {
+        UnaryIter::new(self, pos)
     }
 
     /// Gets `get_bits(pos, 64)` but it can extend further `len()`, padding with zeros.
@@ -540,6 +529,24 @@ impl std::fmt::Debug for BitVector {
     }
 }
 
+impl Searial for BitVector {
+    fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mem = util::vec_io::serialize_usize(&self.words, &mut writer)?;
+        writer.write_u64::<LittleEndian>(self.len as u64)?;
+        Ok(mem + size_of::<u64>())
+    }
+
+    fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let words = util::vec_io::deserialize_usize(&mut reader)?;
+        let len = reader.read_u64::<LittleEndian>()? as usize;
+        Ok(Self { words, len })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        util::vec_io::size_in_bytes(&self.words) + size_of::<u64>()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -559,11 +566,14 @@ mod tests {
     }
 
     fn test_bit_vector(bits: &[bool]) {
-        let bv = BitVector::from_bits(bits);
+        let bv = BitVector::from_bits(bits.iter().cloned());
         assert_eq!(bits.len(), bv.len());
         for i in 0..bits.len() {
             assert_eq!(bits[i], bv.get_bit(i));
         }
+        for (i, x) in bv.iter().enumerate() {
+            assert_eq!(bits[i], x);
+        }
 
         let mut other = BitVector::with_len(bits.len());
         assert_eq!(bv.len(), other.len());
@@ -660,7 +670,7 @@ mod tests {
     #[test]
     fn test_serialize() {
         let mut bytes = vec![];
-        let bv = BitVector::from_bits(&gen_random_bits(10000, 42));
+        let bv = BitVector::from_bits(gen_random_bits(10000, 42));
         let size = bv.serialize_into(&mut bytes).unwrap();
         let other = BitVector::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(bv, other);
