@@ -1,28 +1,16 @@
+//! Unary iterator on bit vectors.
 use crate::bit_vector::WORD_LEN;
 use crate::broadword;
 use crate::BitVector;
 
-/// Iterator for enumerating positions of set bits.
-///
-/// # Examples
-///
-/// ```
-/// use sucds::BitVector;
-///
-/// let bv = BitVector::from_bits(&[false, true, false, false, true]);
-/// let mut it = bv.unary_iter(1);
-///
-/// assert_eq!(it.next(), Some(1));
-/// assert_eq!(it.next(), Some(4));
-/// assert_eq!(it.next(), None);
-/// ```
-pub struct UnaryIterator<'a> {
+/// Iterator for enumerating positions of set bits, created by [`BitVector::unary_iter`].
+pub struct UnaryIter<'a> {
     bv: &'a BitVector,
     pos: usize,
     buf: usize,
 }
 
-impl<'a> UnaryIterator<'a> {
+impl<'a> UnaryIter<'a> {
     /// Creates the iterator from the given bit position.
     pub fn new(bv: &'a BitVector, pos: usize) -> Self {
         let buf =
@@ -43,7 +31,7 @@ impl<'a> UnaryIterator<'a> {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[false, true, false, false, true, true]);
+    /// let bv = BitVector::from_bits([false, true, false, false, true, true]);
     /// let mut it = bv.unary_iter(0);
     ///
     /// assert_eq!(it.skip1(0), Some(1));
@@ -81,7 +69,7 @@ impl<'a> UnaryIterator<'a> {
     /// ```
     /// use sucds::BitVector;
     ///
-    /// let bv = BitVector::from_bits(&[false, true, false, false, true, true]);
+    /// let bv = BitVector::from_bits([false, true, false, false, true, true]);
     /// let mut it = bv.unary_iter(0);
     ///
     /// assert_eq!(it.skip0(0), Some(0));
@@ -114,7 +102,7 @@ impl<'a> UnaryIterator<'a> {
     }
 }
 
-impl<'a> Iterator for UnaryIterator<'a> {
+impl<'a> Iterator for UnaryIter<'a> {
     type Item = usize;
 
     #[inline(always)]
@@ -150,7 +138,7 @@ mod tests {
     }
 
     fn test_unary_iter(bits: &[bool], pos: usize) {
-        let bv = BitVector::from_bits(bits);
+        let bv = BitVector::from_bits(bits.iter().cloned());
 
         let mut expected = vec![];
         for (i, &b) in bits[pos..].iter().enumerate() {
@@ -166,7 +154,7 @@ mod tests {
     }
 
     fn test_skip1(bits: &[bool], mut pos: usize) {
-        let bv = BitVector::from_bits(bits);
+        let bv = BitVector::from_bits(bits.iter().cloned());
         pos = bv.successor1(pos).unwrap();
 
         let mut it = bv.unary_iter(pos);
@@ -178,7 +166,7 @@ mod tests {
     }
 
     fn test_skip0(bits: &[bool], mut pos: usize) {
-        let bv = BitVector::from_bits(bits);
+        let bv = BitVector::from_bits(bits.iter().cloned());
         pos = bv.successor0(pos).unwrap();
 
         let mut it = bv.unary_iter(pos);
