@@ -1,12 +1,16 @@
+//! Constant-time select data structure over integer sets with the dense array technique by Okanohara and Sadakane.
 #![cfg(target_pointer_width = "64")]
 
+pub mod iter;
+
 use std::io::{Read, Write};
 use std::mem::size_of;
 
 use anyhow::Result;
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
 
-use crate::{broadword, util, BitVector};
+use crate::darray::iter::Iter;
+use crate::{broadword, util, BitVector, Searial};
 
 const BLOCK_LEN: usize = 1024;
 const SUBBLOCK_LEN: usize = 32;
@@ -21,17 +25,10 @@ const MAX_IN_BLOCK_DISTANCE: usize = 1 << 16;
 /// ```
 /// use sucds::DArray;
 ///
-/// let da = DArray::from_bits(&[true, false, false, true]);
+/// let da = DArray::from_bits([true, false, false, true]);
 ///
 /// assert_eq!(da.select(0), 0);
 /// assert_eq!(da.select(1), 3);
-///
-/// let mut bytes = vec![];
-/// let size = da.serialize_into(&mut bytes).unwrap();
-/// let other = DArray::deserialize_from(&bytes[..]).unwrap();
-/// assert_eq!(da, other);
-/// assert_eq!(size, bytes.len());
-/// assert_eq!(size, da.size_in_bytes());
 /// ```
 ///
 /// # References
@@ -50,9 +47,9 @@ impl DArray {
     /// # Arguments
     ///
     /// - `bits`: List of bits.
-    pub fn from_bits<'a, I>(bits: I) -> Self
+    pub fn from_bits<I>(bits: I) -> Self
     where
-        I: IntoIterator<Item = &'a bool>,
+        I: IntoIterator<Item = bool>,
     {
         let bv = BitVector::from_bits(bits);
         Self {
@@ -61,33 +58,6 @@ impl DArray {
         }
     }
 
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
-    ///
-    /// # Arguments
-    ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
-        let mem = self.bv.serialize_into(&mut writer)? + self.da.serialize_into(&mut writer)?;
-        Ok(mem)
-    }
-
-    /// Deserializes the data structure from the reader.
-    ///
-    /// # Arguments
-    ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
-        let bv = BitVector::deserialize_from(&mut reader)?;
-        let da = DArrayIndex::deserialize_from(&mut reader)?;
-        Ok(Self { bv, da })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        self.bv.size_in_bytes() + self.da.size_in_bytes()
-    }
-
     /// Searches the `k`-th iteger.
     ///
     /// # Arguments
@@ -103,7 +73,7 @@ impl DArray {
     /// ```
     /// use sucds::DArray;
     ///
-    /// let da = DArray::from_bits(&[true, false, false, true]);
+    /// let da = DArray::from_bits([true, false, false, true]);
     /// assert_eq!(da.select(0), 0);
     /// assert_eq!(da.select(1), 3);
     /// ```
@@ -112,6 +82,24 @@ impl DArray {
         self.da.select(&self.bv, k)
     }
 
+    /// Creates an iterator for enumerating integers.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::DArray;
+    ///
+    /// let da = DArray::from_bits([true, false, false, true]);
+    /// let mut it = da.iter();
+    ///
+    /// assert_eq!(it.next(), Some(0));
+    /// assert_eq!(it.next(), Some(3));
+    /// assert_eq!(it.next(), None);
+    /// ```
+    pub const fn iter(&self) -> Iter {
+        Iter::new(self)
+    }
+
     /// Gets the number of integers.
     #[inline(always)]
     pub const fn len(&self) -> usize {
@@ -125,6 +113,23 @@ impl DArray {
     }
 }
 
+impl Searial for DArray {
+    fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mem = self.bv.serialize_into(&mut writer)? + self.da.serialize_into(&mut writer)?;
+        Ok(mem)
+    }
+
+    fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let bv = BitVector::deserialize_from(&mut reader)?;
+        let da = DArrayIndex::deserialize_from(&mut reader)?;
+        Ok(Self { bv, da })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        self.bv.size_in_bytes() + self.da.size_in_bytes()
+    }
+}
+
 /// The index implementation of [`DArray`] separated from the bit vector.
 #[derive(Default, Debug, Clone, PartialEq, Eq)]
 pub struct DArrayIndex {
@@ -146,50 +151,6 @@ impl DArrayIndex {
         Self::build(bv, over_one)
     }
 
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
-    ///
-    /// # Arguments
-    ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
-        let mut mem = util::vec_io::serialize_isize(&self.block_inventory, &mut writer)?;
-        mem += util::vec_io::serialize_u16(&self.subblock_inventory, &mut writer)?;
-        mem += util::vec_io::serialize_usize(&self.overflow_positions, &mut writer)?;
-        writer.write_u64::<LittleEndian>(self.num_positions as u64)?;
-        writer.write_u8(self.over_one as u8)?;
-        Ok(mem + size_of::<u64>() + size_of::<u8>())
-    }
-
-    /// Deserializes the data structure from the reader.
-    ///
-    /// # Arguments
-    ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
-        let block_inventory = util::vec_io::deserialize_isize(&mut reader)?;
-        let subblock_inventory = util::vec_io::deserialize_u16(&mut reader)?;
-        let overflow_positions = util::vec_io::deserialize_usize(&mut reader)?;
-        let num_positions = reader.read_u64::<LittleEndian>()? as usize;
-        let over_one = reader.read_u8()? != 0;
-        Ok(Self {
-            block_inventory,
-            subblock_inventory,
-            overflow_positions,
-            num_positions,
-            over_one,
-        })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        util::vec_io::size_in_bytes(&self.block_inventory)
-            + util::vec_io::size_in_bytes(&self.subblock_inventory)
-            + util::vec_io::size_in_bytes(&self.overflow_positions)
-            + size_of::<u64>()
-            + size_of::<u8>()
-    }
-
     /// Searches the `k`-th iteger.
     ///
     /// # Arguments
@@ -206,7 +167,7 @@ impl DArrayIndex {
     /// ```
     /// use sucds::{BitVector, darray::DArrayIndex};
     ///
-    /// let bv = BitVector::from_bits(&[true, false, false, true]);
+    /// let bv = BitVector::from_bits([true, false, false, true]);
     /// let da = DArrayIndex::new(&bv, true);
     /// assert_eq!(da.select(&bv, 0), 0);
     /// assert_eq!(da.select(&bv, 1), 3);
@@ -366,6 +327,40 @@ impl DArrayIndex {
     }
 }
 
+impl Searial for DArrayIndex {
+    fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mut mem = util::vec_io::serialize_isize(&self.block_inventory, &mut writer)?;
+        mem += util::vec_io::serialize_u16(&self.subblock_inventory, &mut writer)?;
+        mem += util::vec_io::serialize_usize(&self.overflow_positions, &mut writer)?;
+        writer.write_u64::<LittleEndian>(self.num_positions as u64)?;
+        writer.write_u8(self.over_one as u8)?;
+        Ok(mem + size_of::<u64>() + size_of::<u8>())
+    }
+
+    fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let block_inventory = util::vec_io::deserialize_isize(&mut reader)?;
+        let subblock_inventory = util::vec_io::deserialize_u16(&mut reader)?;
+        let overflow_positions = util::vec_io::deserialize_usize(&mut reader)?;
+        let num_positions = reader.read_u64::<LittleEndian>()? as usize;
+        let over_one = reader.read_u8()? != 0;
+        Ok(Self {
+            block_inventory,
+            subblock_inventory,
+            overflow_positions,
+            num_positions,
+            over_one,
+        })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        util::vec_io::size_in_bytes(&self.block_inventory)
+            + util::vec_io::size_in_bytes(&self.subblock_inventory)
+            + util::vec_io::size_in_bytes(&self.overflow_positions)
+            + size_of::<u64>()
+            + size_of::<u8>()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -401,7 +396,7 @@ mod tests {
 
     #[test]
     fn test_tiny_bits() {
-        let bv = BitVector::from_bits(&[true, false, false, true, false, true, true]);
+        let bv = BitVector::from_bits([true, false, false, true, false, true, true]);
         let da = DArrayIndex::new(&bv, true);
         assert_eq!(da.select(&bv, 0), 0);
         assert_eq!(da.select(&bv, 1), 3);
@@ -416,7 +411,7 @@ mod tests {
     #[test]
     fn test_random_bits_dense() {
         for seed in 0..100 {
-            let bv = BitVector::from_bits(&gen_random_bits(10000, 0.5, seed));
+            let bv = BitVector::from_bits(gen_random_bits(10000, 0.5, seed));
             let da = DArrayIndex::new(&bv, true);
             test_select(&bv, &da);
             let da = DArrayIndex::new(&bv, false);
@@ -427,7 +422,7 @@ mod tests {
     #[test]
     fn test_random_bits_sparse() {
         for seed in 0..100 {
-            let bv = BitVector::from_bits(&gen_random_bits(10000, 0.01, seed));
+            let bv = BitVector::from_bits(gen_random_bits(10000, 0.01, seed));
             let da = DArrayIndex::new(&bv, true);
             test_select(&bv, &da);
             let da = DArrayIndex::new(&bv, false);
@@ -438,7 +433,7 @@ mod tests {
     #[test]
     fn test_serialize_dense() {
         let mut bytes = vec![];
-        let da = DArray::from_bits(&gen_random_bits(10000, 0.5, 42));
+        let da = DArray::from_bits(gen_random_bits(10000, 0.5, 42));
         let size = da.serialize_into(&mut bytes).unwrap();
         let other = DArray::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(da, other);
@@ -449,7 +444,7 @@ mod tests {
     #[test]
     fn test_serialize_sparse() {
         let mut bytes = vec![];
-        let da = DArray::from_bits(&gen_random_bits(10000, 0.01, 42));
+        let da = DArray::from_bits(gen_random_bits(10000, 0.01, 42));
         let size = da.serialize_into(&mut bytes).unwrap();
         let other = DArray::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(da, other);
