@@ -1,3 +1,4 @@
+//! Compressed monotone increasing sequence through Elias-Fano encoding.
 #![cfg(target_pointer_width = "64")]
 
 pub mod iter;
@@ -9,8 +10,8 @@ use std::ops::Range;
 use anyhow::{anyhow, Result};
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
 
-use crate::elias_fano::iter::EliasFanoIterator;
-use crate::{broadword, darray::DArrayIndex, BitVector};
+use crate::elias_fano::iter::Iter;
+use crate::{broadword, darray::DArrayIndex, BitVector, Searial};
 
 const LINEAR_SCAN_THRESHOLD: usize = 64;
 
@@ -26,30 +27,24 @@ const LINEAR_SCAN_THRESHOLD: usize = 64;
 /// # Example
 ///
 /// ```
-/// use sucds::{EliasFano, EliasFanoBuilder};
+/// use sucds::EliasFano;
 ///
-/// let mut b = EliasFanoBuilder::new(10, 4).unwrap();
-/// b.append(&[2, 3, 6, 9]).unwrap();
-///
-/// let ef = EliasFano::new(b, true);
+/// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap();
+/// assert_eq!(ef.len(), 4);
+/// assert_eq!(ef.universe(), 8);
+/// assert_eq!(ef.select(0), 1);
 /// assert_eq!(ef.select(1), 3);
-/// assert_eq!(ef.select(3), 9);
+/// assert_eq!(ef.select(2), 3);
+/// assert_eq!(ef.select(3), 7);
 ///
-/// assert_eq!(ef.rank(5), 2);
-/// assert_eq!(ef.rank(10), 4);
-///
-/// assert_eq!(ef.predecessor(5), Some(3));
-/// assert_eq!(ef.predecessor(1), None);
-///
-/// assert_eq!(ef.successor(4), Some(6));
-/// assert_eq!(ef.successor(10), None);
-///
-/// let mut bytes = vec![];
-/// let size = ef.serialize_into(&mut bytes).unwrap();
-/// let other = EliasFano::deserialize_from(&bytes[..]).unwrap();
-/// assert_eq!(ef, other);
-/// assert_eq!(size, bytes.len());
-/// assert_eq!(size, ef.size_in_bytes());
+/// // Builds an index to enable rank, predecessor, and successor.
+/// let ef = ef.enable_rank();
+/// assert_eq!(ef.rank(3), 1);
+/// assert_eq!(ef.rank(4), 3);
+/// assert_eq!(ef.predecessor(4), Some(3));
+/// assert_eq!(ef.predecessor(3), Some(3));
+/// assert_eq!(ef.successor(3), Some(3));
+/// assert_eq!(ef.successor(4), Some(7));
 /// ```
 ///
 /// # References
@@ -71,50 +66,11 @@ pub struct EliasFano {
 }
 
 impl EliasFano {
-    /// Creates a new [`EliasFano`] from builder `b`.
+    /// Creates a new [`EliasFano`] from a bit stream.
     ///
     /// # Arguments
     ///
-    /// - `b`: Builder.
-    /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use sucds::{EliasFano, EliasFanoBuilder};
-    ///
-    /// let mut b = EliasFanoBuilder::new(10, 4).unwrap();
-    /// b.append(&[2, 3, 6, 9]).unwrap();
-    ///
-    /// let ef = EliasFano::new(b, false);
-    /// assert_eq!(ef.select(0), 2);
-    /// assert_eq!(ef.select(1), 3);
-    /// assert_eq!(ef.select(2), 6);
-    /// assert_eq!(ef.select(3), 9);
-    /// ```
-    pub fn new(b: EliasFanoBuilder, with_rank_index: bool) -> Self {
-        let high_bits_d1 = DArrayIndex::new(&b.high_bits, true);
-        let high_bits_d0 = if with_rank_index {
-            Some(DArrayIndex::new(&b.high_bits, false))
-        } else {
-            None
-        };
-        Self {
-            high_bits: b.high_bits,
-            high_bits_d1,
-            high_bits_d0,
-            low_bits: b.low_bits,
-            low_len: b.low_len,
-            universe: b.universe,
-        }
-    }
-
-    /// Creates a new [`EliasFano`] from input bitset `bits`.
-    ///
-    /// # Arguments
-    ///
-    /// - `bits`: List of bits.
-    /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
+    /// - `bits`: Bit stream.
     ///
     /// # Errors
     ///
@@ -125,43 +81,15 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_bits(&[true, false, false, true], true).unwrap();
-    /// assert_eq!(ef.rank(2), 1);
+    /// let ef = EliasFano::from_bits([true, false, false, true]).unwrap();
+    /// assert_eq!(ef.select(0), 0);
     /// assert_eq!(ef.select(1), 3);
-    /// assert_eq!(ef.predecessor(2), Some(0));
-    /// assert_eq!(ef.successor(1), Some(3));
     /// ```
-    pub fn from_bits<'a, I>(bits: I, with_rank_index: bool) -> Result<Self>
+    pub fn from_bits<I>(bits: I) -> Result<Self>
     where
-        I: IntoIterator<Item = &'a bool>,
+        I: IntoIterator<Item = bool>,
     {
-        Self::from_bitvec(&BitVector::from_bits(bits), with_rank_index)
-    }
-
-    /// Creates a new [`EliasFano`] from input bit vector `bv`.
-    ///
-    /// # Arguments
-    ///
-    /// - `bv`: Input bit vector.
-    /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
-    ///
-    /// # Errors
-    ///
-    /// `anyhow::Error` will be returned if `bv` contains no set bit.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use sucds::{EliasFano, BitVector};
-    ///
-    /// let bv = BitVector::from_bits(&[true, false, false, true]);
-    /// let ef = EliasFano::from_bitvec(&bv, true).unwrap();
-    /// assert_eq!(ef.rank(2), 1);
-    /// assert_eq!(ef.select(1), 3);
-    /// assert_eq!(ef.predecessor(2), Some(0));
-    /// assert_eq!(ef.successor(1), Some(3));
-    /// ```
-    pub fn from_bitvec(bv: &BitVector, with_rank_index: bool) -> Result<Self> {
+        let bv = BitVector::from_bits(bits);
         let n = bv.len();
         let m = (0..bv.num_words())
             .into_iter()
@@ -172,99 +100,57 @@ impl EliasFano {
                 b.push(i)?;
             }
         }
-        Ok(Self::new(b, with_rank_index))
+        Ok(b.build())
     }
 
-    /// Creates a new [`EliasFano`] from monotone-increased integers.
+    /// Creates a new [`EliasFano`] from a list of monotone increasing integers.
     ///
     /// # Arguments
     ///
-    /// - `ints`: Slice of monotone-increased integers .
-    /// - `with_rank_index`: Flag to build the index for rank, predecessor and successor.
+    /// - `ints`: List of monotone increasing integers .
     ///
     /// # Errors
     ///
-    /// `anyhow::Error` will be returned if `ints` is empty or is not monotone-increased.
+    /// `anyhow::Error` will be returned if `ints` is empty or is not monotone increasing.
     ///
     /// # Examples
     ///
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap();
     /// assert_eq!(ef.select(0), 1);
     /// assert_eq!(ef.select(1), 3);
     /// assert_eq!(ef.select(2), 3);
     /// assert_eq!(ef.select(3), 7);
     /// ```
-    pub fn from_ints(ints: &[usize], with_rank_index: bool) -> Result<Self> {
+    pub fn from_ints(ints: &[usize]) -> Result<Self> {
         if ints.is_empty() {
             return Err(anyhow!("The input ints must not be empty."));
         }
         let mut b = EliasFanoBuilder::new(*ints.last().unwrap() + 1, ints.len())?;
         b.append(ints)?;
-        Ok(Self::new(b, with_rank_index))
+        Ok(b.build())
     }
 
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
+    /// Builds an index to enable operations [`EliasFano::rank`],
+    /// [`EliasFano::predecessor`], and [`EliasFano::successor`].
     ///
-    /// # Arguments
+    /// # Examples
     ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
-        let mut mem = self.high_bits.serialize_into(&mut writer)?;
-        mem += self.high_bits_d1.serialize_into(&mut writer)?;
-        if let Some(high_bits_d0) = &self.high_bits_d0 {
-            writer.write_u8(1)?;
-            mem += high_bits_d0.serialize_into(&mut writer)?;
-        } else {
-            writer.write_u8(0)?;
-        }
-        mem += self.low_bits.serialize_into(&mut writer)?;
-        writer.write_u64::<LittleEndian>(self.low_len as u64)?;
-        writer.write_u64::<LittleEndian>(self.universe as u64)?;
-        Ok(mem + size_of::<u8>() + size_of::<u64>() + size_of::<u64>())
-    }
-
-    /// Deserializes the data structure from the reader.
+    /// ```
+    /// use sucds::EliasFano;
     ///
-    /// # Arguments
-    ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
-        let high_bits = BitVector::deserialize_from(&mut reader)?;
-        let high_bits_d1 = DArrayIndex::deserialize_from(&mut reader)?;
-        let high_bits_d0 = if reader.read_u8()? != 0 {
-            Some(DArrayIndex::deserialize_from(&mut reader)?)
-        } else {
-            None
-        };
-        let low_bits = BitVector::deserialize_from(&mut reader)?;
-        let low_len = reader.read_u64::<LittleEndian>()? as usize;
-        let universe = reader.read_u64::<LittleEndian>()? as usize;
-        Ok(Self {
-            high_bits,
-            high_bits_d1,
-            high_bits_d0,
-            low_bits,
-            low_len,
-            universe,
-        })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        self.high_bits.size_in_bytes()
-            + self.high_bits_d1.size_in_bytes()
-            + size_of::<u8>()
-            + self
-                .high_bits_d0
-                .as_ref()
-                .map_or(0, |high_bits_d0| high_bits_d0.size_in_bytes())
-            + self.low_bits.size_in_bytes()
-            + size_of::<u64>()
-            + size_of::<u64>()
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap().enable_rank();
+    /// assert_eq!(ef.rank(1), 0);
+    /// assert_eq!(ef.rank(2), 1);
+    /// assert_eq!(ef.rank(3), 1);
+    /// assert_eq!(ef.rank(4), 3);
+    /// ```
+    #[must_use]
+    pub fn enable_rank(mut self) -> Self {
+        self.high_bits_d0 = Some(DArrayIndex::new(&self.high_bits, false));
+        self
     }
 
     /// Searches the `k`-th iteger.
@@ -282,7 +168,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap();
     /// assert_eq!(ef.select(0), 1);
     /// assert_eq!(ef.select(1), 3);
     /// assert_eq!(ef.select(2), 3);
@@ -309,7 +195,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], true).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap().enable_rank();
     /// assert_eq!(ef.rank(1), 0);
     /// assert_eq!(ef.rank(2), 1);
     /// assert_eq!(ef.rank(3), 1);
@@ -359,7 +245,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], true).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap().enable_rank();
     /// assert_eq!(ef.predecessor(4), Some(3));
     /// assert_eq!(ef.predecessor(3), Some(3));
     /// assert_eq!(ef.predecessor(2), Some(1));
@@ -387,7 +273,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], true).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap().enable_rank();
     /// assert_eq!(ef.successor(0), Some(1));
     /// assert_eq!(ef.successor(2), Some(3));
     /// assert_eq!(ef.successor(3), Some(3));
@@ -416,7 +302,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap();
     /// assert_eq!(ef.delta(0), 1);
     /// assert_eq!(ef.delta(1), 2);
     /// assert_eq!(ef.delta(2), 0);
@@ -452,7 +338,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10]).unwrap();
     /// assert_eq!(ef.find(6), Some(3));
     /// assert_eq!(ef.find(10), Some(5));
     /// assert_eq!(ef.find(9), None);
@@ -480,7 +366,7 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10]).unwrap();
     /// assert_eq!(ef.find_range(1..4, 6), Some(3));
     /// assert_eq!(ef.find_range(5..6, 10), Some(5));
     /// assert_eq!(ef.find_range(1..3, 6), None);
@@ -517,7 +403,7 @@ impl EliasFano {
         None
     }
 
-    /// Creates an iterator of [`EliasFanoIterator`] to enumerate integers from the `k`-th one.
+    /// Creates an iterator of [`Iter`] to enumerate integers from the `k`-th one.
     ///
     /// # Arguments
     ///
@@ -528,15 +414,15 @@ impl EliasFano {
     /// ```
     /// use sucds::EliasFano;
     ///
-    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7], false).unwrap();
+    /// let ef = EliasFano::from_ints(&[1, 3, 3, 7]).unwrap();
     /// let mut it = ef.iter(1);
     /// assert_eq!(it.next(), Some(3));
     /// assert_eq!(it.next(), Some(3));
     /// assert_eq!(it.next(), Some(7));
     /// assert_eq!(it.next(), None);
     /// ```
-    pub fn iter(&self, k: usize) -> EliasFanoIterator {
-        EliasFanoIterator::new(self, k)
+    pub fn iter(&self, k: usize) -> Iter {
+        Iter::new(self, k)
     }
 
     /// Gets the number of integers.
@@ -558,6 +444,57 @@ impl EliasFano {
     }
 }
 
+impl Searial for EliasFano {
+    fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mut mem = self.high_bits.serialize_into(&mut writer)?;
+        mem += self.high_bits_d1.serialize_into(&mut writer)?;
+        if let Some(high_bits_d0) = &self.high_bits_d0 {
+            writer.write_u8(1)?;
+            mem += high_bits_d0.serialize_into(&mut writer)?;
+        } else {
+            writer.write_u8(0)?;
+        }
+        mem += self.low_bits.serialize_into(&mut writer)?;
+        writer.write_u64::<LittleEndian>(self.low_len as u64)?;
+        writer.write_u64::<LittleEndian>(self.universe as u64)?;
+        Ok(mem + size_of::<u8>() + size_of::<u64>() + size_of::<u64>())
+    }
+
+    fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let high_bits = BitVector::deserialize_from(&mut reader)?;
+        let high_bits_d1 = DArrayIndex::deserialize_from(&mut reader)?;
+        let high_bits_d0 = if reader.read_u8()? != 0 {
+            Some(DArrayIndex::deserialize_from(&mut reader)?)
+        } else {
+            None
+        };
+        let low_bits = BitVector::deserialize_from(&mut reader)?;
+        let low_len = reader.read_u64::<LittleEndian>()? as usize;
+        let universe = reader.read_u64::<LittleEndian>()? as usize;
+        Ok(Self {
+            high_bits,
+            high_bits_d1,
+            high_bits_d0,
+            low_bits,
+            low_len,
+            universe,
+        })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        self.high_bits.size_in_bytes()
+            + self.high_bits_d1.size_in_bytes()
+            + size_of::<u8>()
+            + self
+                .high_bits_d0
+                .as_ref()
+                .map_or(0, |high_bits_d0| high_bits_d0.size_in_bytes())
+            + self.low_bits.size_in_bytes()
+            + size_of::<u64>()
+            + size_of::<u64>()
+    }
+}
+
 /// Builder of [`EliasFano`].
 pub struct EliasFanoBuilder {
     high_bits: BitVector,
@@ -609,16 +546,16 @@ impl EliasFanoBuilder {
     /// # Example
     ///
     /// ```
-    /// use sucds::{EliasFano, EliasFanoBuilder};
+    /// use sucds::EliasFanoBuilder;
     ///
-    /// let mut b = EliasFanoBuilder::new(10, 4).unwrap();
-    /// [2, 3, 6, 9].iter().for_each(|&x| b.push(x).unwrap());
+    /// let mut b = EliasFanoBuilder::new(8, 4).unwrap();
+    /// [1, 3, 3, 7].iter().for_each(|&x| b.push(x).unwrap());
     ///
-    /// let ef = EliasFano::new(b, false);
-    /// assert_eq!(ef.select(0), 2);
+    /// let ef = b.build();
+    /// assert_eq!(ef.select(0), 1);
     /// assert_eq!(ef.select(1), 3);
-    /// assert_eq!(ef.select(2), 6);
-    /// assert_eq!(ef.select(3), 9);
+    /// assert_eq!(ef.select(2), 3);
+    /// assert_eq!(ef.select(3), 7);
     /// ```
     pub fn push(&mut self, i: usize) -> Result<()> {
         if i < self.last {
@@ -666,16 +603,16 @@ impl EliasFanoBuilder {
     /// # Example
     ///
     /// ```
-    /// use sucds::{EliasFano, EliasFanoBuilder};
+    /// use sucds::EliasFanoBuilder;
     ///
-    /// let mut b = EliasFanoBuilder::new(10, 4).unwrap();
-    /// b.append(&[2, 3, 6, 9]).unwrap();
+    /// let mut b = EliasFanoBuilder::new(8, 4).unwrap();
+    /// b.append(&[1, 3, 3, 7]).unwrap();
     ///
-    /// let ef = EliasFano::new(b, false);
-    /// assert_eq!(ef.select(0), 2);
+    /// let ef = b.build();
+    /// assert_eq!(ef.select(0), 1);
     /// assert_eq!(ef.select(1), 3);
-    /// assert_eq!(ef.select(2), 6);
-    /// assert_eq!(ef.select(3), 9);
+    /// assert_eq!(ef.select(2), 3);
+    /// assert_eq!(ef.select(3), 7);
     /// ```
     pub fn append<'a, I>(&mut self, ints: I) -> Result<()>
     where
@@ -686,6 +623,33 @@ impl EliasFanoBuilder {
         }
         Ok(())
     }
+
+    /// Builds [`EliasFano`] from the pushed integers.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::EliasFanoBuilder;
+    ///
+    /// let mut b = EliasFanoBuilder::new(8, 4).unwrap();
+    /// b.append(&[1, 3, 3, 7]).unwrap();
+    ///
+    /// let ef = b.build();
+    /// assert_eq!(ef.select(0), 1);
+    /// assert_eq!(ef.select(1), 3);
+    /// assert_eq!(ef.select(2), 3);
+    /// assert_eq!(ef.select(3), 7);
+    /// ```
+    pub fn build(self) -> EliasFano {
+        EliasFano {
+            high_bits_d1: DArrayIndex::new(&self.high_bits, true),
+            high_bits: self.high_bits,
+            high_bits_d0: None,
+            low_bits: self.low_bits,
+            low_len: self.low_len,
+            universe: self.universe,
+        }
+    }
 }
 
 #[cfg(test)]
@@ -792,7 +756,7 @@ mod tests {
 
     #[test]
     fn test_tiny_bits() {
-        let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10], false).unwrap();
+        let ef = EliasFano::from_ints(&[1, 3, 3, 6, 7, 10]).unwrap();
         assert_eq!(ef.select(0), 1);
         assert_eq!(ef.select(1), 3);
         assert_eq!(ef.select(2), 3);
@@ -819,7 +783,9 @@ mod tests {
     fn test_random_bits_dense() {
         for seed in 0..100 {
             let bits = gen_random_bits(10000, 0.5, seed);
-            let ef = EliasFano::from_bits(&bits, true).unwrap();
+            let ef = EliasFano::from_bits(bits.iter().cloned())
+                .unwrap()
+                .enable_rank();
             test_rank_select(&bits, &ef);
             test_successor_predecessor(&bits, &ef);
             let queries = gen_random_queries(&bits, 100, seed + 100);
@@ -833,7 +799,9 @@ mod tests {
     fn test_random_bits_sparse() {
         for seed in 0..100 {
             let bits = gen_random_bits(10000, 0.01, seed);
-            let ef = EliasFano::from_bits(&bits, true).unwrap();
+            let ef = EliasFano::from_bits(bits.iter().cloned())
+                .unwrap()
+                .enable_rank();
             test_rank_select(&bits, &ef);
             test_successor_predecessor(&bits, &ef);
             let queries = gen_random_queries(&bits, 100, seed + 100);
@@ -846,7 +814,9 @@ mod tests {
     #[test]
     fn test_serialize_dense() {
         let mut bytes = vec![];
-        let ef = EliasFano::from_bits(&gen_random_bits(10000, 0.5, 42), true).unwrap();
+        let ef = EliasFano::from_bits(gen_random_bits(10000, 0.5, 42))
+            .unwrap()
+            .enable_rank();
         let size = ef.serialize_into(&mut bytes).unwrap();
         let other = EliasFano::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(ef, other);
@@ -857,7 +827,9 @@ mod tests {
     #[test]
     fn test_serialize_sparse() {
         let mut bytes = vec![];
-        let ef = EliasFano::from_bits(&gen_random_bits(10000, 0.01, 42), true).unwrap();
+        let ef = EliasFano::from_bits(gen_random_bits(10000, 0.01, 42))
+            .unwrap()
+            .enable_rank();
         let size = ef.serialize_into(&mut bytes).unwrap();
         let other = EliasFano::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(ef, other);
