@@ -1,10 +1,14 @@
+//! Compressed integer list with prefix-summed Elias-Fano encoding.
 #![cfg(target_pointer_width = "64")]
 
+pub mod iter;
+
 use std::io::{Read, Write};
 
 use anyhow::Result;
 
-use crate::{EliasFano, EliasFanoBuilder};
+use crate::elias_fano_list::iter::Iter;
+use crate::{EliasFano, EliasFanoBuilder, Searial};
 
 /// Compressed integer list with prefix-summed Elias-Fano encoding.
 ///
@@ -28,13 +32,6 @@ use crate::{EliasFano, EliasFanoBuilder};
 ///
 /// assert_eq!(list.len(), 4);
 /// assert_eq!(list.sum(), 31);
-///
-/// let mut bytes = vec![];
-/// let size = list.serialize_into(&mut bytes).unwrap();
-/// let other = EliasFanoList::deserialize_from(&bytes[..]).unwrap();
-/// assert_eq!(list, other);
-/// assert_eq!(size, bytes.len());
-/// assert_eq!(size, list.size_in_bytes());
 /// ```
 #[derive(Default, Debug, Clone, PartialEq, Eq)]
 pub struct EliasFanoList {
@@ -69,34 +66,7 @@ impl EliasFanoList {
             cur += x;
             b.push(cur)?;
         }
-        Ok(Self {
-            ef: EliasFano::new(b, false),
-        })
-    }
-
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
-    ///
-    /// # Arguments
-    ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, writer: W) -> Result<usize> {
-        self.ef.serialize_into(writer)
-    }
-
-    /// Deserializes the data structure from the reader.
-    ///
-    /// # Arguments
-    ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(reader: R) -> Result<Self> {
-        let ef = EliasFano::deserialize_from(reader)?;
-        Ok(Self { ef })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        self.ef.size_in_bytes()
+        Ok(Self { ef: b.build() })
     }
 
     /// Gets the `i`-th integer.
@@ -124,6 +94,26 @@ impl EliasFanoList {
         self.ef.delta(i)
     }
 
+    /// Creates an iterator for enumerating integers.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::EliasFanoList;
+    ///
+    /// let list = EliasFanoList::from_slice(&[5, 14, 2, 10]).unwrap();
+    /// let mut it = list.iter();
+    ///
+    /// assert_eq!(it.next(), Some(5));
+    /// assert_eq!(it.next(), Some(14));
+    /// assert_eq!(it.next(), Some(2));
+    /// assert_eq!(it.next(), Some(10));
+    /// assert_eq!(it.next(), None);
+    /// ```
+    pub const fn iter(&self) -> Iter {
+        Iter::new(self)
+    }
+
     /// Gets the number of integers.
     pub const fn len(&self) -> usize {
         self.ef.len()
@@ -140,6 +130,21 @@ impl EliasFanoList {
     }
 }
 
+impl Searial for EliasFanoList {
+    fn serialize_into<W: Write>(&self, writer: W) -> Result<usize> {
+        self.ef.serialize_into(writer)
+    }
+
+    fn deserialize_from<R: Read>(reader: R) -> Result<Self> {
+        let ef = EliasFano::deserialize_from(reader)?;
+        Ok(Self { ef })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        self.ef.size_in_bytes()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -160,6 +165,9 @@ mod tests {
         }
         assert_eq!(ints.len(), list.len());
         assert_eq!(acc, list.sum());
+        for (i, x) in list.iter().enumerate() {
+            assert_eq!(ints[i], x);
+        }
     }
 
     #[test]
