@@ -1,3 +1,4 @@
+//! Rank/select data structure over bit vectors with Vigna's rank9 and hinted selection techniques.
 #![cfg(target_pointer_width = "64")]
 
 use std::io::{Read, Write};
@@ -6,7 +7,7 @@ use std::mem::size_of;
 use anyhow::Result;
 use byteorder::{ReadBytesExt, WriteBytesExt};
 
-use crate::{broadword, util, BitVector};
+use crate::{broadword, util, BitVector, Searial};
 
 const BLOCK_LEN: usize = 8;
 const SELECT_ONES_PER_HINT: usize = 64 * BLOCK_LEN * 2;
@@ -26,20 +27,14 @@ const SELECT_ZEROS_PER_HINT: usize = SELECT_ONES_PER_HINT;
 /// ```
 /// use sucds::RsBitVector;
 ///
-/// let bv = RsBitVector::from_bits(&[true, false, false, true], true, true);
+/// let bv = RsBitVector::from_bits([true, false, false, true]).select1_hints().select0_hints();
 ///
 /// assert_eq!(bv.get_bit(1), false);
 /// assert_eq!(bv.rank1(1), 1);
 /// assert_eq!(bv.rank0(1), 0);
 /// assert_eq!(bv.select1(1), 3);
 /// assert_eq!(bv.select0(0), 1);
-///
-/// let mut bytes = vec![];
-/// let size = bv.serialize_into(&mut bytes).unwrap();
-/// let other = RsBitVector::deserialize_from(&bytes[..]).unwrap();
-/// assert_eq!(bv, other);
-/// assert_eq!(size, bytes.len());
-/// assert_eq!(size, bv.size_in_bytes());
+/// assert_eq!(bv.len(), 4);
 /// ```
 ///
 /// # References
@@ -59,30 +54,21 @@ impl RsBitVector {
     /// # Arguments
     ///
     /// - `bv`: Input bit vector.
-    /// - `select1_hints`: Flag to build the index for select1.
-    /// - `select0_hints`: Flag to build the index for select0.
     ///
     /// # Examples
     ///
     /// ```
     /// use sucds::{BitVector, RsBitVector};
     ///
-    /// let bv = RsBitVector::new(BitVector::from_bits(&[true, false, false, true]), true, true);
+    /// let bv = RsBitVector::new(BitVector::from_bits([true, false, false, true]));
+    /// assert_eq!(bv.get_bit(0), true);
     /// assert_eq!(bv.get_bit(1), false);
-    /// assert_eq!(bv.rank1(1), 1);
-    /// assert_eq!(bv.rank0(1), 0);
-    /// assert_eq!(bv.select1(1), 3);
-    /// assert_eq!(bv.select0(0), 1);
+    /// assert_eq!(bv.get_bit(2), false);
+    /// assert_eq!(bv.get_bit(3), true);
+    /// assert_eq!(bv.len(), 4);
     /// ```
-    pub fn new(bv: BitVector, select1_hints: bool, select0_hints: bool) -> Self {
-        let mut this = Self::build_rank(bv);
-        if select1_hints {
-            this = this.build_select0();
-        }
-        if select0_hints {
-            this = this.build_select1();
-        }
-        this
+    pub fn new(bv: BitVector) -> Self {
+        Self::build_rank(bv)
     }
 
     /// Creates a new [`RsBitVector`] from input bitset `bits`.
@@ -90,90 +76,56 @@ impl RsBitVector {
     /// # Arguments
     ///
     /// - `bits`: List of bits.
-    /// - `select1_hints`: Flag to build the index for select1.
-    /// - `select0_hints`: Flag to build the index for select0.
     ///
     /// # Examples
     ///
     /// ```
     /// use sucds::RsBitVector;
     ///
-    /// let bv = RsBitVector::from_bits(&[true, false, false, true], true, true);
+    /// let bv = RsBitVector::from_bits([true, false, false, true]);
+    /// assert_eq!(bv.get_bit(0), true);
     /// assert_eq!(bv.get_bit(1), false);
-    /// assert_eq!(bv.rank1(1), 1);
-    /// assert_eq!(bv.rank0(1), 0);
-    /// assert_eq!(bv.select1(1), 3);
-    /// assert_eq!(bv.select0(0), 1);
+    /// assert_eq!(bv.get_bit(2), false);
+    /// assert_eq!(bv.get_bit(3), true);
+    /// assert_eq!(bv.len(), 4);
     /// ```
-    pub fn from_bits<'a, I>(bits: I, select1_hints: bool, select0_hints: bool) -> Self
+    pub fn from_bits<I>(bits: I) -> Self
     where
-        I: IntoIterator<Item = &'a bool>,
+        I: IntoIterator<Item = bool>,
     {
-        Self::new(BitVector::from_bits(bits), select1_hints, select0_hints)
+        Self::new(BitVector::from_bits(bits))
     }
 
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
+    /// Builds an index for faster `select1`.
     ///
-    /// # Arguments
+    /// # Examples
     ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
-        let mut mem = self.bv.serialize_into(&mut writer)?;
-        mem += util::vec_io::serialize_usize(&self.block_rank_pairs, &mut writer)?;
-        if let Some(select1_hints) = &self.select1_hints {
-            writer.write_u8(1)?;
-            mem += util::vec_io::serialize_usize(select1_hints, &mut writer)?;
-        } else {
-            writer.write_u8(0)?;
-        }
-        if let Some(select0_hints) = &self.select0_hints {
-            writer.write_u8(1)?;
-            mem += util::vec_io::serialize_usize(select0_hints, &mut writer)?;
-        } else {
-            writer.write_u8(0)?;
-        }
-        Ok(mem + size_of::<u8>() * 2)
+    /// ```
+    /// use sucds::RsBitVector;
+    ///
+    /// let bv = RsBitVector::from_bits([true, false, false, true]).select1_hints();
+    /// assert_eq!(bv.select1(0), 0);
+    /// assert_eq!(bv.select1(1), 3);
+    /// ```
+    #[must_use]
+    pub fn select1_hints(self) -> Self {
+        self.build_select1()
     }
 
-    /// Deserializes the data structure from the reader.
+    /// Builds an index for faster `select0`.
     ///
-    /// # Arguments
+    /// # Examples
     ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
-        let bv = BitVector::deserialize_from(&mut reader)?;
-        let block_rank_pairs = util::vec_io::deserialize_usize(&mut reader)?;
-        let select1_hints = if reader.read_u8()? != 0 {
-            Some(util::vec_io::deserialize_usize(&mut reader)?)
-        } else {
-            None
-        };
-        let select0_hints = if reader.read_u8()? != 0 {
-            Some(util::vec_io::deserialize_usize(&mut reader)?)
-        } else {
-            None
-        };
-        Ok(Self {
-            bv,
-            block_rank_pairs,
-            select1_hints,
-            select0_hints,
-        })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        self.bv.size_in_bytes()
-            + util::vec_io::size_in_bytes(&self.block_rank_pairs)
-            + size_of::<u8>()
-            + self.select1_hints.as_ref().map_or(0, |select1_hints| {
-                util::vec_io::size_in_bytes(select1_hints)
-            })
-            + size_of::<u8>()
-            + self.select0_hints.as_ref().map_or(0, |select0_hints| {
-                util::vec_io::size_in_bytes(select0_hints)
-            })
+    /// ```
+    /// use sucds::RsBitVector;
+    ///
+    /// let bv = RsBitVector::from_bits([true, false, false, true]).select0_hints();
+    /// assert_eq!(bv.select0(0), 1);
+    /// assert_eq!(bv.select0(1), 2);
+    /// ```
+    #[must_use]
+    pub fn select0_hints(self) -> Self {
+        self.build_select0()
     }
 
     /// Gets the `pos`-th bit.
@@ -187,7 +139,7 @@ impl RsBitVector {
     /// ```
     /// use sucds::RsBitVector;
     ///
-    /// let bv = RsBitVector::from_bits(&[true, false, false, true], false, false);
+    /// let bv = RsBitVector::from_bits([true, false, false, true]);
     /// assert_eq!(bv.get_bit(0), true);
     /// assert_eq!(bv.get_bit(1), false);
     /// assert_eq!(bv.get_bit(2), false);
@@ -213,7 +165,7 @@ impl RsBitVector {
     /// ```
     /// use sucds::RsBitVector;
     ///
-    /// let bv = RsBitVector::from_bits(&[true, false, false, true], false, false);
+    /// let bv = RsBitVector::from_bits([true, false, false, true]);
     /// assert_eq!(bv.rank1(1), 1);
     /// assert_eq!(bv.rank1(2), 1);
     /// assert_eq!(bv.rank1(3), 1);
@@ -248,7 +200,7 @@ impl RsBitVector {
     /// ```
     /// use sucds::RsBitVector;
     ///
-    /// let bv = RsBitVector::from_bits(&[true, false, false, true], false, false);
+    /// let bv = RsBitVector::from_bits([true, false, false, true]);
     /// assert_eq!(bv.rank0(1), 0);
     /// assert_eq!(bv.rank0(2), 1);
     /// assert_eq!(bv.rank0(3), 2);
@@ -274,7 +226,7 @@ impl RsBitVector {
     /// ```
     /// use sucds::RsBitVector;
     ///
-    /// let bv = RsBitVector::from_bits(&[true, false, false, true], true, false);
+    /// let bv = RsBitVector::from_bits([true, false, false, true]).select1_hints();
     /// assert_eq!(bv.select1(0), 0);
     /// assert_eq!(bv.select1(1), 3);
     /// ```
@@ -335,7 +287,7 @@ impl RsBitVector {
     /// ```
     /// use sucds::RsBitVector;
     ///
-    /// let bv = RsBitVector::from_bits(&[true, false, false, true], false, true);
+    /// let bv = RsBitVector::from_bits([true, false, false, true]).select0_hints();
     /// assert_eq!(bv.select0(0), 1);
     /// assert_eq!(bv.select0(1), 2);
     /// ```
@@ -381,6 +333,11 @@ impl RsBitVector {
         word_offset * 64 + broadword::select_in_word(!self.bv.words()[word_offset], k - cur_rank)
     }
 
+    /// Gets the reference of the internal bit vector.
+    pub const fn bit_vector(&self) -> &BitVector {
+        &self.bv
+    }
+
     /// Gets the number of bits.
     #[inline(always)]
     pub const fn len(&self) -> usize {
@@ -512,6 +469,60 @@ impl RsBitVector {
     }
 }
 
+impl Searial for RsBitVector {
+    fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mut mem = self.bv.serialize_into(&mut writer)?;
+        mem += util::vec_io::serialize_usize(&self.block_rank_pairs, &mut writer)?;
+        if let Some(select1_hints) = &self.select1_hints {
+            writer.write_u8(1)?;
+            mem += util::vec_io::serialize_usize(select1_hints, &mut writer)?;
+        } else {
+            writer.write_u8(0)?;
+        }
+        if let Some(select0_hints) = &self.select0_hints {
+            writer.write_u8(1)?;
+            mem += util::vec_io::serialize_usize(select0_hints, &mut writer)?;
+        } else {
+            writer.write_u8(0)?;
+        }
+        Ok(mem + size_of::<u8>() * 2)
+    }
+
+    fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let bv = BitVector::deserialize_from(&mut reader)?;
+        let block_rank_pairs = util::vec_io::deserialize_usize(&mut reader)?;
+        let select1_hints = if reader.read_u8()? != 0 {
+            Some(util::vec_io::deserialize_usize(&mut reader)?)
+        } else {
+            None
+        };
+        let select0_hints = if reader.read_u8()? != 0 {
+            Some(util::vec_io::deserialize_usize(&mut reader)?)
+        } else {
+            None
+        };
+        Ok(Self {
+            bv,
+            block_rank_pairs,
+            select1_hints,
+            select0_hints,
+        })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        self.bv.size_in_bytes()
+            + util::vec_io::size_in_bytes(&self.block_rank_pairs)
+            + size_of::<u8>()
+            + self.select1_hints.as_ref().map_or(0, |select1_hints| {
+                util::vec_io::size_in_bytes(select1_hints)
+            })
+            + size_of::<u8>()
+            + self.select0_hints.as_ref().map_or(0, |select0_hints| {
+                util::vec_io::size_in_bytes(select0_hints)
+            })
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -552,7 +563,9 @@ mod tests {
     fn test_random_bits() {
         for seed in 0..100 {
             let bits = gen_random_bits(10000, seed);
-            let bv = RsBitVector::from_bits(&bits, true, true);
+            let bv = RsBitVector::from_bits(bits.iter().cloned())
+                .select1_hints()
+                .select0_hints();
             test_rank_select1(&bits, &bv);
             test_rank_select0(&bits, &bv);
         }
@@ -561,7 +574,9 @@ mod tests {
     #[test]
     fn test_serialize() {
         let mut bytes = vec![];
-        let bv = RsBitVector::from_bits(&gen_random_bits(10000, 42), true, true);
+        let bv = RsBitVector::from_bits(gen_random_bits(10000, 42))
+            .select1_hints()
+            .select0_hints();
         let size = bv.serialize_into(&mut bytes).unwrap();
         let other = RsBitVector::deserialize_from(&bytes[..]).unwrap();
         assert_eq!(bv, other);
