@@ -1,5 +1,9 @@
+//! Time- and space-efficient data structure for a sequence of integers,
+//! supporting some queries such as ranking, selection, and intersection.
 #![cfg(target_pointer_width = "64")]
 
+pub mod iter;
+
 use std::io::{Read, Write};
 use std::mem::size_of;
 use std::ops::Range;
@@ -7,7 +11,8 @@ use std::ops::Range;
 use anyhow::{anyhow, Result};
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
 
-use crate::{broadword, BitVector, RsBitVector};
+use crate::wavelet_matrix::iter::Iter;
+use crate::{broadword, BitVector, RsBitVector, Searial};
 
 /// Time- and space-efficient data structure for a sequence of integers,
 /// supporting some queries such as ranking, selection, and intersection.
@@ -24,7 +29,7 @@ use crate::{broadword, BitVector, RsBitVector};
 /// use sucds::WaveletMatrix;
 ///
 /// let text = "tobeornottobethatisthequestion";
-/// let wm = WaveletMatrix::from_text(text).unwrap();
+/// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
 ///
 /// assert_eq!(wm.len(), text.chars().count());
 /// assert_eq!(wm.dim(), 'u' as usize + 1);
@@ -32,13 +37,6 @@ use crate::{broadword, BitVector, RsBitVector};
 /// assert_eq!(wm.get(20), 'h' as usize);
 /// assert_eq!(wm.rank(22, 'o' as usize), 4);
 /// assert_eq!(wm.select(2, 't' as usize), 9);
-///
-/// let mut bytes = vec![];
-/// let size = wm.serialize_into(&mut bytes).unwrap();
-/// let other = WaveletMatrix::deserialize_from(&bytes[..]).unwrap();
-/// assert_eq!(wm, other);
-/// assert_eq!(size, bytes.len());
-/// assert_eq!(size, wm.size_in_bytes());
 /// ```
 ///
 /// # References
@@ -49,7 +47,7 @@ pub struct WaveletMatrix {
     layers: Vec<RsBitVector>,
     dim: usize,
     len: usize,
-    bit_length: usize,
+    width: usize,
 }
 
 impl WaveletMatrix {
@@ -65,134 +63,23 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let ints = vec![20, 7, 13, 2, 11];
-    /// let wm = WaveletMatrix::from_ints(&ints).unwrap();
+    /// let wm = WaveletMatrix::from_ints(ints.iter().cloned()).unwrap();
     ///
     /// assert_eq!(wm.get(2), 13);
     /// assert_eq!(wm.len(), ints.len());
     /// assert_eq!(wm.dim(), 21);
     /// ```
-    pub fn from_ints<'a, I>(ints: I) -> Result<Self>
+    pub fn from_ints<I>(ints: I) -> Result<Self>
     where
-        I: IntoIterator<Item = &'a usize>,
+        I: IntoIterator<Item = usize>,
     {
         let mut wmb = WaveletMatrixBuilder::default();
-        for &v in ints {
+        for v in ints {
             wmb.push(v);
         }
         wmb.build()
     }
 
-    /// Builds a [`WaveletMatrix`] from an iterator of bytes.
-    ///
-    /// # Arguments
-    ///
-    /// - `bytes`: Iterator of bytes.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use sucds::WaveletMatrix;
-    ///
-    /// let bytes = b"abaabb";
-    /// let wm = WaveletMatrix::from_bytes(bytes).unwrap();
-    ///
-    /// assert_eq!(wm.get(2), b'a' as usize);
-    /// assert_eq!(wm.len(), bytes.len());
-    /// assert_eq!(wm.dim(), b'b' as usize + 1);
-    /// ```
-    pub fn from_bytes<'a, I>(bytes: I) -> Result<Self>
-    where
-        I: IntoIterator<Item = &'a u8>,
-    {
-        let mut wmb = WaveletMatrixBuilder::default();
-        for &v in bytes {
-            wmb.push(v as usize);
-        }
-        wmb.build()
-    }
-
-    /// Builds a [`WaveletMatrix`] from characters in a string.
-    /// Note that this handles a sequence of `char`, not `u8`.
-    ///
-    /// # Arguments
-    ///
-    /// - `text`: String.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use sucds::WaveletMatrix;
-    ///
-    /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
-    ///
-    /// assert_eq!(wm.get(20), 'h' as usize);
-    /// assert_eq!(wm.len(), text.chars().count());
-    /// assert_eq!(wm.dim(), 'u' as usize + 1);
-    /// ```
-    pub fn from_text<S>(text: S) -> Result<Self>
-    where
-        S: AsRef<str>,
-    {
-        let mut wmb = WaveletMatrixBuilder::default();
-        for c in text.as_ref().chars() {
-            wmb.push(c as usize);
-        }
-        wmb.build()
-    }
-
-    /// Serializes the data structure into the writer,
-    /// returning the number of serialized bytes.
-    ///
-    /// # Arguments
-    ///
-    /// - `writer`: `std::io::Write` variable.
-    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
-        let mut mem = 0;
-        writer.write_u64::<LittleEndian>(self.layers.len() as u64)?;
-        for layer in &self.layers {
-            mem += layer.serialize_into(&mut writer)?;
-        }
-        writer.write_u64::<LittleEndian>(self.dim as u64)?;
-        writer.write_u64::<LittleEndian>(self.len as u64)?;
-        writer.write_u64::<LittleEndian>(self.bit_length as u64)?;
-        Ok(mem + size_of::<u64>() * 4)
-    }
-
-    /// Deserializes the data structure from the reader.
-    ///
-    /// # Arguments
-    ///
-    /// - `reader`: `std::io::Read` variable.
-    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
-        let layers = {
-            let len = reader.read_u64::<LittleEndian>()? as usize;
-            let mut layers = Vec::with_capacity(len);
-            for _ in 0..len {
-                layers.push(RsBitVector::deserialize_from(&mut reader)?);
-            }
-            layers
-        };
-        let dim = reader.read_u64::<LittleEndian>()? as usize;
-        let len = reader.read_u64::<LittleEndian>()? as usize;
-        let bit_length = reader.read_u64::<LittleEndian>()? as usize;
-        Ok(Self {
-            layers,
-            dim,
-            len,
-            bit_length,
-        })
-    }
-
-    /// Returns the number of bytes to serialize the data structure.
-    pub fn size_in_bytes(&self) -> usize {
-        let mut mem = 0;
-        for layer in &self.layers {
-            mem += layer.size_in_bytes();
-        }
-        mem + size_of::<u64>() * 4
-    }
-
     /// Gets the maximum value + 1 in stored integers.
     #[inline(always)]
     pub const fn dim(&self) -> usize {
@@ -211,10 +98,10 @@ impl WaveletMatrix {
         self.len() == 0
     }
 
-    /// Gets the maximum numbber of bits needed to be stored for each integers.
+    /// Gets the maximum number of bits needed to be stored for each integers.
     #[inline(always)]
-    pub const fn bit_length(&self) -> usize {
-        self.bit_length
+    pub const fn width(&self) -> usize {
+        self.width
     }
 
     /// Gets the integer located at `pos`.
@@ -233,7 +120,7 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
+    /// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
     ///
     /// assert_eq!(wm.get(2), 'b' as usize);
     /// assert_eq!(wm.get(5), 'r' as usize);
@@ -242,7 +129,7 @@ impl WaveletMatrix {
     #[inline(always)]
     pub fn get(&self, mut pos: usize) -> usize {
         let mut val = 0;
-        for depth in 0..self.bit_length as usize {
+        for depth in 0..self.width as usize {
             val <<= 1;
             let rsbv = &self.layers[depth];
             if rsbv.get_bit(pos) {
@@ -272,7 +159,7 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
+    /// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
     ///
     /// assert_eq!(wm.rank(14, 'b' as usize), 2);
     /// assert_eq!(wm.rank(14, 'o' as usize), 4);
@@ -300,7 +187,7 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
+    /// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
     ///
     /// assert_eq!(wm.rank_range(0..14, 'o' as usize), 4);
     /// assert_eq!(wm.rank_range(14..20, 'a' as usize), 1);
@@ -310,8 +197,8 @@ impl WaveletMatrix {
     pub fn rank_range(&self, range: Range<usize>, val: usize) -> usize {
         let mut start_pos = range.start;
         let mut end_pos = range.end;
-        for depth in 0..self.bit_length as usize {
-            let bit = Self::get_msb(val, depth, self.bit_length);
+        for depth in 0..self.width as usize {
+            let bit = Self::get_msb(val, depth, self.width);
             let rsbv = &self.layers[depth];
             if bit {
                 start_pos = rsbv.rank1(start_pos) + rsbv.num_zeros();
@@ -341,7 +228,7 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
+    /// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
     ///
     /// assert_eq!(wm.select(2, 't' as usize), 9);
     /// assert_eq!(wm.select(0, 'q' as usize), 22);
@@ -353,10 +240,10 @@ impl WaveletMatrix {
 
     #[inline]
     fn select_helper(&self, mut k: usize, val: usize, mut pos: usize, depth: usize) -> usize {
-        if depth == self.bit_length {
+        if depth == self.width {
             return pos + k;
         }
-        let bit = Self::get_msb(val, depth, self.bit_length);
+        let bit = Self::get_msb(val, depth, self.width);
         let rsbv = &self.layers[depth];
         if bit {
             let zeros = rsbv.num_zeros();
@@ -387,7 +274,7 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
+    /// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
     ///
     /// assert_eq!(wm.quantile(0..5, 0), 'b' as usize); // The zero-th in "tobeo" should be "b"
     /// assert_eq!(wm.quantile(0..5, 1), 'e' as usize); // The first in "tobeo" should be "e"
@@ -402,7 +289,7 @@ impl WaveletMatrix {
         let mut val = 0;
         let mut start_pos = range.start;
         let mut end_pos = range.end;
-        for depth in 0..self.bit_length {
+        for depth in 0..self.width {
             val <<= 1;
             let rsbv = &self.layers[depth];
             let zero_start_pos = rsbv.rank0(start_pos);
@@ -434,7 +321,7 @@ impl WaveletMatrix {
     /// use sucds::WaveletMatrix;
     ///
     /// let text = "tobeornottobethatisthequestion";
-    /// let wm = WaveletMatrix::from_text(text).unwrap();
+    /// let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
     ///
     /// assert_eq!(wm.intersect(&[0..3, 4..5, 10..12], 2), vec!['b' as usize, 'o' as usize]); // "tob", "o", "ob"
     /// assert_eq!(wm.intersect(&[0..3, 4..5, 10..12], 3), vec!['o' as usize]); // "tob", "o", "ob"
@@ -453,13 +340,13 @@ impl WaveletMatrix {
         depth: usize,
         prefix: usize,
     ) -> Vec<usize> {
-        if depth == self.bit_length {
+        if depth == self.width {
             return vec![prefix];
         }
 
         let rsbv = &self.layers[depth];
-        let mut zero_ranges = Vec::new();
-        let mut one_ranges = Vec::new();
+        let mut zero_ranges = vec![];
+        let mut one_ranges = vec![];
         for range in ranges {
             let start_pos = range.start;
             let end_pos = range.end;
@@ -475,7 +362,7 @@ impl WaveletMatrix {
             }
         }
 
-        let mut ret = Vec::new();
+        let mut ret = vec![];
         if zero_ranges.len() >= k {
             ret.extend_from_slice(&self.intersect_helper(&zero_ranges, k, depth + 1, prefix << 1));
         }
@@ -491,8 +378,70 @@ impl WaveletMatrix {
     }
 
     #[inline(always)]
-    const fn get_msb(val: usize, pos: usize, bit_length: usize) -> bool {
-        ((val >> (bit_length - pos - 1)) & 1) == 1
+    const fn get_msb(val: usize, pos: usize, width: usize) -> bool {
+        ((val >> (width - pos - 1)) & 1) == 1
+    }
+
+    /// Creates an iterator for enumerating integers.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::WaveletMatrix;
+    ///
+    /// let wm = WaveletMatrix::from_ints([20, 7, 13, 2]).unwrap();
+    /// let mut it = wm.iter();
+    ///
+    /// assert_eq!(it.next(), Some(20));
+    /// assert_eq!(it.next(), Some(7));
+    /// assert_eq!(it.next(), Some(13));
+    /// assert_eq!(it.next(), Some(2));
+    /// assert_eq!(it.next(), None);
+    /// ```
+    pub const fn iter(&self) -> Iter {
+        Iter::new(self)
+    }
+}
+
+impl Searial for WaveletMatrix {
+    fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mut mem = 0;
+        writer.write_u64::<LittleEndian>(self.layers.len() as u64)?;
+        for layer in &self.layers {
+            mem += layer.serialize_into(&mut writer)?;
+        }
+        writer.write_u64::<LittleEndian>(self.dim as u64)?;
+        writer.write_u64::<LittleEndian>(self.len as u64)?;
+        writer.write_u64::<LittleEndian>(self.width as u64)?;
+        Ok(mem + size_of::<u64>() * 4)
+    }
+
+    fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let layers = {
+            let len = reader.read_u64::<LittleEndian>()? as usize;
+            let mut layers = Vec::with_capacity(len);
+            for _ in 0..len {
+                layers.push(RsBitVector::deserialize_from(&mut reader)?);
+            }
+            layers
+        };
+        let dim = reader.read_u64::<LittleEndian>()? as usize;
+        let len = reader.read_u64::<LittleEndian>()? as usize;
+        let width = reader.read_u64::<LittleEndian>()? as usize;
+        Ok(Self {
+            layers,
+            dim,
+            len,
+            width,
+        })
+    }
+
+    fn size_in_bytes(&self) -> usize {
+        let mut mem = 0;
+        for layer in &self.layers {
+            mem += layer.size_in_bytes();
+        }
+        mem + size_of::<u64>() * 4
     }
 }
 
@@ -504,7 +453,7 @@ pub struct WaveletMatrixBuilder {
 impl WaveletMatrixBuilder {
     /// Creates a new [`WaveletMatrixBuilder`].
     pub const fn new() -> Self {
-        Self { vals: Vec::new() }
+        Self { vals: vec![] }
     }
 
     /// Pusheds integer `val` at the end
@@ -557,43 +506,47 @@ impl WaveletMatrixBuilder {
     /// assert_eq!(wm.rank(22, 'o' as usize), 4);
     /// assert_eq!(wm.select(2, 't' as usize), 9);
     /// ```
-    pub fn build(&self) -> Result<WaveletMatrix> {
+    pub fn build(self) -> Result<WaveletMatrix> {
         if self.vals.is_empty() {
             return Err(anyhow!("vals must not be empty"));
         }
 
-        let dim = self.get_dim()?;
-        let bit_length = Self::get_bit_length(dim)?;
-        let mut zeros: Vec<usize> = self.vals.clone();
-        let mut ones: Vec<usize> = Vec::new();
-        let mut layers: Vec<RsBitVector> = Vec::new();
-        for depth in 0..bit_length {
-            let mut next_zeros: Vec<usize> = Vec::with_capacity(self.vals.len());
-            let mut next_ones: Vec<usize> = Vec::with_capacity(self.vals.len());
+        let len = self.vals.len();
+        let dim = self.get_dim();
+        let width = Self::get_width(dim);
+
+        let mut zeros = self.vals;
+        let mut ones = vec![];
+        let mut layers = vec![];
+
+        for depth in 0..width {
+            let mut next_zeros = Vec::with_capacity(len);
+            let mut next_ones = Vec::with_capacity(len);
             let mut bv = BitVector::new();
             Self::filter(
                 &zeros,
-                bit_length - depth - 1,
+                width - depth - 1,
                 &mut next_zeros,
                 &mut next_ones,
                 &mut bv,
             );
             Self::filter(
                 &ones,
-                bit_length - depth - 1,
+                width - depth - 1,
                 &mut next_zeros,
                 &mut next_ones,
                 &mut bv,
             );
             zeros = next_zeros;
             ones = next_ones;
-            layers.push(RsBitVector::new(bv, true, true));
+            layers.push(RsBitVector::new(bv).select1_hints().select0_hints());
         }
+
         Ok(WaveletMatrix {
             layers,
             dim,
-            len: self.vals.len(),
-            bit_length,
+            len,
+            width,
         })
     }
 
@@ -604,29 +557,23 @@ impl WaveletMatrixBuilder {
         next_ones: &mut Vec<usize>,
         bv: &mut BitVector,
     ) {
-        for val in vals {
-            let bit = ((*val >> shift) & 1) == 1;
+        for &val in vals {
+            let bit = ((val >> shift) & 1) == 1;
             bv.push_bit(bit);
             if bit {
-                next_ones.push(*val);
+                next_ones.push(val);
             } else {
-                next_zeros.push(*val);
+                next_zeros.push(val);
             }
         }
     }
 
-    fn get_dim(&self) -> Result<usize> {
-        match self.vals.iter().max() {
-            Some(i) => Ok(*i + 1),
-            None => Err(anyhow!("cannot extract max value")),
-        }
+    fn get_dim(&self) -> usize {
+        *self.vals.iter().max().unwrap() + 1
     }
 
-    fn get_bit_length(val: usize) -> Result<usize> {
-        match broadword::msb(val) {
-            Some(i) => Ok(i + 1),
-            None => Err(anyhow!("cannot calculate msb")),
-        }
+    fn get_width(val: usize) -> usize {
+        broadword::msb(val).unwrap() + 1
     }
 }
 
@@ -667,6 +614,12 @@ mod test {
         }
     }
 
+    fn test_iter(ints: &[usize], wm: &WaveletMatrix) {
+        for (i, x) in wm.iter().enumerate() {
+            assert_eq!(ints[i], x);
+        }
+    }
+
     fn test_rank_select(ints: &[usize], wm: &WaveletMatrix, val: usize) {
         let mut cur_rank = 0;
         for i in 0..ints.len() {
@@ -746,7 +699,7 @@ mod test {
     fn test_builder_push() {
         let mut wmb = WaveletMatrixBuilder::new();
         wmb.push(123);
-        assert_eq!(wmb.get_dim().unwrap(), 124);
+        assert_eq!(wmb.get_dim(), 124);
     }
 
     #[test]
@@ -755,7 +708,7 @@ mod test {
         wmb.push(123);
         wmb.push(7777);
         wmb.push(987);
-        assert_eq!(wmb.get_dim().unwrap(), 7778);
+        assert_eq!(wmb.get_dim(), 7778);
     }
 
     #[test]
@@ -773,7 +726,7 @@ mod test {
         let text = "tobeornottobethatisthequestion";
         let len = text.chars().count();
 
-        let wm = WaveletMatrix::from_text(text).unwrap();
+        let wm = WaveletMatrix::from_ints(text.chars().map(|c| c as usize)).unwrap();
         assert_eq!(wm.len(), len);
         assert_eq!(wm.dim(), ('u' as usize) + 1);
 
@@ -795,8 +748,9 @@ mod test {
         let ints = gen_random_ints(1000, 13);
         let ranges = gen_random_ranges(30, ints.len(), 13);
 
-        let wm = WaveletMatrix::from_ints(&ints).unwrap();
+        let wm = WaveletMatrix::from_ints(ints.iter().cloned()).unwrap();
         test_lookup(&ints, &wm);
+        test_iter(&ints, &wm);
         for val in 0..SIGMA {
             test_rank_select(&ints, &wm, val);
             test_rank_range(&ints, &wm, &ranges, val);
@@ -804,4 +758,15 @@ mod test {
         test_quantile(&ints, &wm, &ranges);
         test_intersect(&ints, &wm, &ranges);
     }
+
+    #[test]
+    fn test_serialize() {
+        let mut bytes = vec![];
+        let wm = WaveletMatrix::from_ints(gen_random_ints(1000, 13)).unwrap();
+        let size = wm.serialize_into(&mut bytes).unwrap();
+        let other = WaveletMatrix::deserialize_from(&bytes[..]).unwrap();
+        assert_eq!(wm, other);
+        assert_eq!(size, bytes.len());
+        assert_eq!(size, wm.size_in_bytes());
+    }
 }
